import Project._
import lcf._
import fol._

object Tests {

  def checkNormalFormTheorem(f: Formula): Boolean = {
    val nf = toNormalForm(f)
    lazy val prettyF = f.pretty
    toNormalFormThm(f).formula match {
      case Iff(f1, f2) =>
        assert(f1 == f, s"The left-hand-side of `toNormalFormThm($prettyF)` should be $prettyF. Got ${f1.pretty} instead.")
        assert(f2 == nf, s"The right-hand side of `toNormalFormThm($prettyF)` should be ${nf.pretty}. Got ${f2.pretty} instead.")
        assert(isNormalForm(nf), s"The normal form of $f should be in normal form. Got ${nf.pretty} instead.")
      case _ =>
        throw new Exception(s"`toNormalFormThm($prettyF)` should return an `Iff` formula")
    }
    true
  }

  def testThese[T](toTest: Iterable[T], toString: T => String, f: T => Boolean): Boolean = {
    for (e <- toTest) {
      println("Testing: " + toString(e))
      if (f(e))
        println(s"  ==> OK")
      else {
        println(s"  ==> Error")
        return false
      }
    }
    return true
  }

  def testRandom[T](n: Int, toString: T => String, random: => T, f: T => Boolean): Boolean = {
    for (i <- 1 to n) {
      if (!f(random)) {
        return false
      }
      if (i % 1000 == 0)
        println(s"Did $i random (successful) tests.")
    }
    return true
  }
}
